import { Injectable } from '@nestjs/common';
import { BaseDataService } from 'src/core/modules/data/service/base-data.service';
import { SalesPriceFormulaEntity } from '../../domain/entities/sales-price-formula.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { SalesPriceFormulaModel } from '../models/sales-price-formula.model';
import { CONNECTION_NAME } from 'src/core/strings/constants/base.constants';
import { Repository } from 'typeorm';
import { FormulaType } from '../../constants';
import { TaxModel } from 'src/modules/transaction/tax/data/models/tax.model';

@Injectable()
export class SalesPriceFormulaDataService extends BaseDataService<SalesPriceFormulaEntity> {
  constructor(
    @InjectRepository(SalesPriceFormulaModel, CONNECTION_NAME.DEFAULT)
    private repo: Repository<SalesPriceFormulaModel>,
    @InjectRepository(TaxModel, CONNECTION_NAME.DEFAULT)
    private tax: Repository<TaxModel>,
  ) {
    super(repo);
  }

  profitShareFormula() {
    return this.repo.find({
      where: {
        type: FormulaType.PROFIT_SHARE,
      },
    });
  }

  async salesPriceFormula() {
    const salesFormula = await this.repo.findOne({
      where: {
        type: FormulaType.SALES_PRICE,
      },
    });

    const taxes = await this.tax.find();

    for (const tax of taxes) {
      salesFormula.formula_string = salesFormula.formula_string.replace(
        tax.name,
        `(${tax.formula_string})`,
      );
    }

    return salesFormula;
  }
}
